@@ -18,9 +18,10 @@ from sqlalchemy.orm.exc import NoResultFound
 from .access import AccessControl
 from .account_settings import AccountSettings
 from .const import Constants
+from .database import db_abort
 from .grants import Grants
 from .service_config import ServiceConfig
-from . import geocode, singletons, utils
+from . import geocode, state, utils
 
 
 class BasicCRUD(object):
@@ -41,14 +42,14 @@ class BasicCRUD(object):
     """
 
     def __init__(self, resource=None, model=None):
-        self.models = ServiceConfig().models
+        self.models = state.models
         self.resource = resource
-        if self.resource not in singletons.controller:
+        if self.resource not in state.controllers:
             if model:
                 self.model = model
             else:
                 self.model = getattr(self.models, resource.capitalize())
-            singletons.controller[self.resource] = self
+            state.controllers[self.resource] = self
 
     @staticmethod
     def create(body, id_prefix='x-', limit_related={}):
@@ -69,7 +70,7 @@ class BasicCRUD(object):
             response code (201 on success)
         """
 
-        self = singletons.controller[request.url_rule.rule.split('/')[3]]
+        self = state.controllers[request.url_rule.rule.split('/')[3]]
         # TODO reject extraneous keys beyond those in the openapi path spec
         if self.resource == 'contact':
             retval = self._create_contact(body)
@@ -92,14 +93,12 @@ class BasicCRUD(object):
                     body['expires'], '%Y-%m-%dT%H:%M:%SZ')
         if hasattr(self.model, 'uid') and not body.get('uid'):
             body['uid'] = acc.uid
-        uid = body.get('uid')
+        uid = logmsg['uid'] = body.get('uid')
         if not acc.with_permission('c', new_uid=uid,
-                                   membership=acc.primary_res,
+                                   membership=acc.primary_resource,
                                    id=body.get('event_id')):
-            logging.warning(dict(
-                message='access denied', uid=uid, **logmsg))
+            logging.warning(dict(message='access denied', **logmsg))
             return dict(message=_(u'access denied')), 403
-        logmsg['uid'] = body.get('uid')
         ret_info = {}
         logging.info(dict(id=id, name=body.get('name'), **logmsg))
         if not body.get('category_id') and hasattr(self.model, 'category_id'):
@@ -115,10 +114,11 @@ class BasicCRUD(object):
         if hasattr(self.model, 'status'):
             body['status'] = body.get('status', 'active')
 
-        limit = Grants().get(self.resource if self.resource.endswith('s')
-                             else self.resource + 's', uid=uid)
-        if limit and g.db.query(self.model).filter_by(
-                uid=uid).count() >= limit:
+        grant = (self.resource if self.resource.endswith('s')
+                 else self.resource + 's')
+        limit = Grants().get(grant, uid=uid)
+        if grant in ServiceConfig().config.DEFAULT_GRANTS and g.db.query(
+                self.model).filter_by(uid=uid).count() >= limit:
             msg = _('user limit exceeded')
             logging.info(dict(message=msg, allowed=limit, **logmsg))
             return dict(message=msg, allowed=limit), 405
@@ -155,8 +155,7 @@ class BasicCRUD(object):
             logging.warning(dict(message=message, error=str(ex), **logmsg))
             return dict(message=message, data=str(body)), 405
         except Exception as ex:
-            logging.warning(dict(message=str(ex), **logmsg))
-            return dict(message=str(ex), data=str(body)), 405
+            return db_abort(str(ex), rollback=True, **logmsg)
         return dict(id=id, **ret_info), 201
 
     @staticmethod
@@ -174,7 +173,7 @@ class BasicCRUD(object):
             message, second element is response code (200 on success)
         """
 
-        self = singletons.controller[request.url_rule.rule.split('/')[3]]
+        self = state.controllers[request.url_rule.rule.split('/')[3]]
         acc = AccessControl(model=self.model)
         try:
             query = self.db_get(id)
@@ -191,9 +190,9 @@ class BasicCRUD(object):
         if 'modified' in retval and not retval['modified']:
             del(retval['modified'])
         eid = None if hasattr(self.model, 'event_id') else (
-            acc.auth_ids[acc.primary_res] or [None])[0]
+            acc.auth_ids[acc.primary_resource] or [None])[0]
         retval['rbac'] = ''.join(sorted(list(
-            acc.rbac_permissions(query=query, membership=acc.primary_res,
+            acc.rbac_permissions(query=query, membership=acc.primary_resource,
                                  id=eid) - set('c'))))
         if 'geolat' in retval:
             access = 'r' if 'r' in retval['rbac'] else None
@@ -227,7 +226,7 @@ class BasicCRUD(object):
         if 'id' in body and body['id'] != id:
             return dict(message='id is a read-only property',
                         title='Bad Request'), 405
-        self = singletons.controller[request.url_rule.rule.split('/')[3]]
+        self = state.controllers[request.url_rule.rule.split('/')[3]]
         body['modified'] = utils.utcnow()
         if body.get('expires'):
             try:
@@ -236,6 +235,8 @@ class BasicCRUD(object):
             except ValueError:
                 body['expires'] = datetime.strptime(
                     body['expires'], '%Y-%m-%dT%H:%M:%SZ')
+        if 'totp' in body and not body.pop('totp'):
+            body['totp_secret'] = None
         if hasattr(self.model, '__rest_related__'):
             for related in self.model.__rest_related__:
                 if related in body:
@@ -261,8 +262,7 @@ class BasicCRUD(object):
             logging.warning(dict(message=message, error=str(ex), **logmsg))
             return dict(message=message, data=str(ex)), 405
         except Exception as ex:
-            logging.warning(dict(message=str(ex), **logmsg))
-            return dict(message='exception %s' % str(ex)), 405
+            return db_abort(str(ex), rollback=True, **logmsg)
 
         updated = {key: body[key] for key, val in body.items()
                    if key in current and current[key] != val and
@@ -287,7 +287,7 @@ class BasicCRUD(object):
         """
 
         # TODO - update auth if model could affect any session's auth
-        self = singletons.controller[request.url_rule.rule.split('/')[3]]
+        self = state.controllers[request.url_rule.rule.split('/')[3]]
         logmsg = dict(action='delete', resource=self.resource,
                       account_id=AccessControl().account_id,
                       ident=AccessControl().identity)
@@ -310,14 +310,12 @@ class BasicCRUD(object):
                     logging.info(dict(id=id, status='disabled', **logmsg))
                     query.update(dict(status='disabled'))
             except Exception as ex:
-                logging.warning(dict(message=str(ex), **logmsg))
-                return dict(message='exception %s' % str(ex)), 405
+                return db_abort(str(ex), **logmsg)
             count += 1
         try:
             g.db.commit()
         except Exception as ex:
-            logging.warning(dict(message=str(ex), **logmsg))
-            return dict(message='exception %s' % str(ex)), 405
+            return db_abort(str(ex), rollback=True, **logmsg)
         logging.info(dict(count=count, ids=ids, **logmsg))
         return NoContent, 404 if errors else 204
 
@@ -339,10 +337,15 @@ class BasicCRUD(object):
           dict: items (list), count(int), cursor_next (str)
         """
 
-        self = singletons.controller[request.url_rule.rule.split('/')[3]]
+        self = state.controllers[request.url_rule.rule.split('/')[3]]
         acc = AccessControl(model=self.model)
         logmsg = dict(action='find', ident=acc.identity,
                       resource=self.resource)
+        # TODO this conditional is a band-aid to give status 403 instead of
+        # 200 with empty item list for anonymous / restricted users
+        if self.resource != 'account' and not acc.with_permission(
+               'r', new_uid=acc.uid):
+            return dict(message=_(u'access denied')), 403
         conditions = {item: value for item, value in kwargs.items()
                       if item in ('status')}
         if 'cursor_next' in kwargs:
@@ -413,8 +416,7 @@ class BasicCRUD(object):
         try:
             results = query.slice(offset, offset + limit + 1).all()
         except Exception as ex:
-            logging.error(dict(message=str(ex), **logmsg))
-            return dict(message='Backend problem--probable bug'), 405
+            return db_abort(str(ex), **logmsg)
         retval = dict(items=[], count=query.count())
         count = 0
         for result in results[:limit]:
@@ -427,7 +429,7 @@ class BasicCRUD(object):
             # TODO - takes 30ms per record for rbac: optimize or redesign
             record['rbac'] = ''.join(sorted(list(
                 acc.rbac_permissions(owner_uid=record.get('uid'),
-                                     membership=acc.primary_res,
+                                     membership=acc.primary_resource,
                                      id=record.get('id'),
                                      privacy=record.get('privacy'))
                 - set('c'))))
@@ -459,7 +461,7 @@ class BasicCRUD(object):
                 return dict(message=_(u'invalid mobile number')), 405
             body['info'] = re.sub('[- ()]', '', body['info'])
         elif body.get('type') == 'email':
-            body['info'] = body['info'].strip().lower()
+            body['info'] = utils.identity_normalize(body['info'].strip())
             if not re.match(Constants.REGEX_EMAIL, body['info']):
                 return dict(message=_(u'invalid email address')), 405
         elif 'type' in body and body.get('type') not in ['sms', 'email']:
@@ -492,13 +494,13 @@ class BasicCRUD(object):
                 return dict(message=_(u'invalid mobile number')), 405
             body['info'] = re.sub('[- ()]', '', body['info'])
         elif body.get('type') == 'email':
-            body['info'] = body['info'].strip().lower()
+            body['info'] = utils.identity_normalize(body['info'].strip())
             if not re.match(Constants.REGEX_EMAIL, body['info']):
                 return dict(message=_(u'invalid email address')), 405
         if 'id' in body and body['id'] != id:
             return dict(message='id is a read-only property',
                         title='Bad Request'), 405
-        self = singletons.controller[request.url_rule.rule.split('/')[3]]
+        self = state.controllers[request.url_rule.rule.split('/')[3]]
         body['modified'] = utils.utcnow()
         try:
             query = g.db.query(self.model).filter_by(id=id)
