@@ -11,9 +11,11 @@ from flask_babel import _
 class Constants:
     """Constants for apicrud methods"""
     AUTH_INVITEE = 'invitee'
+    CACHE_SIZE = 16384
     DEFAULT_AWS_REGION = 'us-east-2'
     LIB_MOD_SPATIALITE = ['/usr/lib/x86_64-linux-gnu/mod_spatialite.so',
                           '/usr/lib/mod_spatialite.so.7']
+    MFA_BACKUP_CODELEN = 8
     MIME_IMAGE_TYPES = ("gif", "heic", "jpeg", "png", "svg")
     MIME_VIDEO_TYPES = ("mp4", "mpeg")
     PER_PAGE_DEFAULT = 100
@@ -21,6 +23,7 @@ class Constants:
     REGEX_EMAIL = r'[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,}'
     REGEX_PHONE = r'[0-9()+ -]{5,20}'
     SERVICE_CONFIG_FILE = 'service_config.yaml'
+    TOTP_COOKIE = '__Secure-totp'
 
 
 class i18nValues:
